==> 0.2.0.tar.gz: bin/sampler.dart <==
// Copyright 2014 The Flutter Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'dart:convert';
import 'dart:io' show exitCode, stderr;

import 'package:convert/convert.dart';
import 'package:crypto/crypto.dart';
import 'package:file/file.dart';
import 'package:path/path.dart' as path;
import 'package:platform/platform.dart';
import 'package:process_runner/process_runner.dart';
import 'package:snippets/snippets.dart';

ProcessRunner processRunner = ProcessRunner();

File findExecutableForPlatform(String executableName, Directory packageDirectory) {
  final FileSystem filesystem = FlutterInformation.instance.filesystem;
  switch (FlutterInformation.instance.platform.operatingSystem) {
    case 'linux':
      return filesystem.file(path.join(
        packageDirectory.path,
        'build',
        'linux',
        'x64',
        'release',
        'bundle',
        executableName,
      ));
    case 'macos':
      return filesystem.file(path.joinAll(<String>[
        packageDirectory.path,
        'build',
        'macos',
        'Build',
        'Product',
        'Release',
        '$executableName.app',
        'Contents',
        'MacOS',
        executableName,
      ]));
    case 'windows':
      return filesystem.file(path.joinAll(<String>[
        packageDirectory.path,
        'build',
        'windows',
        'runner',
        'Release',
        '$executableName.exe',
      ]));
    default:
      throw FileSystemException(
          'Unable to find $executableName on platform ${FlutterInformation.instance.platform.operatingSystem}');
  }
}

String getDigests(List<File> files) {
  final AccumulatorSink<Digest> sums = AccumulatorSink<Digest>();
  final ByteConversionSink input = sha1.startChunkedConversion(sums);
  for (final File file in files) {
    sums.add(sha1.convert(file.readAsBytesSync()));
  }
  input.close();
  return sums.events.toString();
}

String getPackageDigest(Directory packageLocation) {
  return getDigests(<File>[
    ...packageLocation
        .childDirectory('lib')
        .listSync(recursive: true)
        .where((FileSystemEntity entity) => entity is File && entity.basename.endsWith('.dart'))
        .cast<File>(),
    packageLocation.childDirectory('bin').childFile('sampler.dart'),
    packageLocation.childFile('pubspec.yaml'),
  ]);
}

Future<File?> buildExecutableIfNeeded(Directory packageLocation) async {
  final Platform platform = FlutterInformation.instance.platform;
  final Directory flutterRoot = FlutterInformation.instance.getFlutterRoot();
  final File mainLocation = packageLocation.childDirectory('lib').childFile('main.dart');
  final File flutterExe =
      flutterRoot.childDirectory('bin').childFile(platform.isWindows ? 'flutter.bat' : 'flutter');
  final File builtExecutable = findExecutableForPlatform('sampler', packageLocation);

  final File lastBuildDigestFile =
      packageLocation.childDirectory('build').childFile('package_digest');
  final String packageDigest = getPackageDigest(packageLocation);
  final String previousDigest =
      lastBuildDigestFile.existsSync() ? lastBuildDigestFile.readAsStringSync() : '';
  if (!builtExecutable.existsSync() || packageDigest != previousDigest) {
    stderr.writeln('Building sampler...');
    stderr.writeln(
        'Due to the way sampler is packaged, this may take a while (~30s) the first time.');
    final ProcessRunnerResult result = await processRunner.runProcess(<String>[
      flutterExe.absolute.path,
      'build',
      platform.operatingSystem,
      '--release',
      '--target=${mainLocation.absolute.path}',
    ], workingDirectory: packageLocation.absolute);
    if (result.exitCode != 0) {
      exitCode = result.exitCode;
      stderr.writeln('Unable to build sampler executable:\n${result.output}');
      return null;
    }
    lastBuildDigestFile.writeAsStringSync(packageDigest);
    stderr.writeln('Sampler built, running $builtExecutable');
  }
  return builtExecutable;
}

Future<void> main(List<String> args) async {
  final Platform platform = FlutterInformation.instance.platform;
  final Directory packageLocation =
      FlutterInformation.instance.filesystem.file(platform.script.toFilePath()).parent.parent;

  final File? builtExecutable = await buildExecutableIfNeeded(packageLocation);
  if (builtExecutable == null) {
    return;
  }

  final ProcessRunnerResult result = await processRunner.runProcess(<String>[
    builtExecutable.absolute.path,
    ...args,
  ], workingDirectory: packageLocation.absolute);

  exitCode = result.exitCode;
}

==> 0.2.3.tar.gz: bin/sampler.dart <==
// Copyright 2014 The Flutter Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'dart:io' show exitCode, stderr;

import 'package:crypto/crypto.dart';
import 'package:file/file.dart';
import 'package:path/path.dart' as path;
import 'package:platform/platform.dart';
import 'package:process_runner/process_runner.dart';
import 'package:snippets/snippets.dart';

ProcessRunner processRunner = ProcessRunner();

File findExecutableForPlatform(String executableName, Directory packageDirectory) {
  final FileSystem filesystem = FlutterInformation.instance.filesystem;
  switch (FlutterInformation.instance.platform.operatingSystem) {
    case 'linux':
      return filesystem.file(path.join(
        packageDirectory.path,
        'build',
        'linux',
        'x64',
        'release',
        'bundle',
        executableName,
      ));
    case 'macos':
      return filesystem.file(path.joinAll(<String>[
        packageDirectory.path,
        'build',
        'macos',
        'Build',
        'Products',
        'Release',
        '$executableName.app',
        'Contents',
        'MacOS',
        executableName,
      ]));
    case 'windows':
      return filesystem.file(path.joinAll(<String>[
        packageDirectory.path,
        'build',
        'windows',
        'runner',
        'Release',
        '$executableName.exe',
      ]));
    default:
      throw FileSystemException('Unable to find $executableName on platform '
          '${FlutterInformation.instance.platform.operatingSystem}');
  }
}

String getDigests(List<File> files) {
  final List<String> contents = <String>[];
  for (final File file in files) {
    contents.add(sha1.convert(file.readAsBytesSync()).toString());
  }
  return sha1.convert(contents.join('').codeUnits).toString();
}

String getPackageDigest(Directory packageLocation) {
  return getDigests(<File>[
    ...packageLocation
        .childDirectory('lib')
        .listSync(recursive: true)
        .where((FileSystemEntity entity) => entity is File && entity.basename.endsWith('.dart'))
        .cast<File>(),
    packageLocation.childDirectory('bin').childFile('sampler.dart'),
    packageLocation.childFile('pubspec.yaml'),
  ]);
}

Future<Directory> copyPackageToTmpDir(
  Directory packageLocation,
  String packageDigest,
  String previousDigest,
) async {
  final Directory tmpDir = FlutterInformation.instance.filesystem.systemTempDirectory
      .childDirectory('sampler_build_${packageDigest.substring(0, 10)}');
  if (tmpDir.existsSync()) {
    return tmpDir;
  }
  if (previousDigest.isNotEmpty) {
    final Directory previousTmpDir = FlutterInformation.instance.filesystem.systemTempDirectory
        .childDirectory('sampler_build_${previousDigest.substring(0, 10)}');
    if (previousTmpDir.existsSync()) {
      try {
        previousTmpDir.deleteSync(recursive: true);
      } on FileSystemException {
        // Don't worry about failures removing the old temporary directory.
      }
    }
  }
  switch (FlutterInformation.instance.platform.operatingSystem) {
    case 'macos':
    case 'linux':
      await processRunner.runProcess(<String>[
        '/bin/cp',
        '-rp',
        packageLocation.absolute.path,
        tmpDir.absolute.path,
      ]);
      break;
    case 'windows':
      await processRunner.runProcess(<String>[
        'xcopy',
        packageLocation.absolute.path,
        tmpDir.absolute.path,
        '/E',
        '/H',
        '/C',
        '/I',
      ]);
      break;
  }
  return tmpDir;
}

Future<File?> buildExecutableIfNeeded(Directory packageLocation) async {
  final Platform platform = FlutterInformation.instance.platform;
  final Directory flutterRoot = FlutterInformation.instance.getFlutterRoot();
  final File flutterExe =
      flutterRoot.childDirectory('bin').childFile(platform.isWindows ? 'flutter.bat' : 'flutter');
  final File lastBuildDigestFile = packageLocation.childFile('package_digest');
  final String packageDigest = getPackageDigest(packageLocation);
  final String previousDigest =
      lastBuildDigestFile.existsSync() ? lastBuildDigestFile.readAsStringSync() : '';

  // Because the flutter tool refuses to run pub get inside of a pub cache directory
  // (ok, that is actually kind of sensible), copy this package into a temp directory
  // and build it there. If the hash changes, then blow away the old temp directory
  // and build it in a new one.
  final Directory tmpDir =
      await copyPackageToTmpDir(packageLocation, packageDigest, previousDigest);
  final File builtExecutable = findExecutableForPlatform('sampler', tmpDir);
  if (!builtExecutable.existsSync() || packageDigest != previousDigest) {
    stderr.writeln('Building sampler...');
    stderr.writeln(
        'Due to the way sampler is packaged, this may take a while (~30s) the first time.');
    final File mainLocation = tmpDir.childDirectory('lib').childFile('main.dart');
    ProcessRunnerResult result = await processRunner.runProcess(<String>[
      flutterExe.absolute.path,
      'create',
      '--overwrite',
      '--project-name=sampler',
      '--org=flutter.dev',
      '--platforms=${platform.operatingSystem}',
      '.',
    ], workingDirectory: tmpDir);
    if (result.exitCode != 0) {
      exitCode = result.exitCode;
      stderr.writeln('Unable to create sampler executable:\n${result.output}');
      return null;
    }

    result = await processRunner.runProcess(<String>[
      flutterExe.absolute.path,
      'build',
      platform.operatingSystem,
      '--release',
      '--target=${mainLocation.absolute.path}',
    ], workingDirectory: tmpDir);
    if (result.exitCode != 0) {
      exitCode = result.exitCode;
      stderr.writeln('Unable to build sampler executable:\n${result.output}');
      return null;
    }
    lastBuildDigestFile.writeAsStringSync(packageDigest);
    stderr.writeln('Sampler built, running $builtExecutable');
  }
  return builtExecutable;
}

Future<void> main(List<String> args) async {
  final Platform platform = FlutterInformation.instance.platform;
  final Directory packageLocation =
      FlutterInformation.instance.filesystem.file(platform.script.toFilePath()).parent.parent;

  final File? builtExecutable = await buildExecutableIfNeeded(packageLocation);
  if (builtExecutable == null) {
    return;
  }

  final ProcessRunnerResult result = await processRunner.runProcess(<String>[
    builtExecutable.absolute.path,
    ...args,
  ], workingDirectory: packageLocation.absolute);

  exitCode = result.exitCode;
}
